from django.db import models
from django.core.validators import MinValueValidator,MaxValueValidator
from .voucher_code_manager import VoucherCodeManager
from clmainApp.models import User,Branch
from brpatientApp.models import Patient
from brsessionApp.models import PatientSession

DISCOUNT_TYPE_CHOICES=(
    ("P", "نسبة مئوية"),#Percentage
    ("F", "مبلغ محدد"),#Fixed Amount
)
class Voucher(models.Model):
    # class DiscountType(models.TextChoices):
    #     PERCENTAGE = 'percentage', 'Percentage'
    #     FIXED_AMOUNT = 'fixed_amount', 'Fixed Amount'
    branch = models.ForeignKey(Branch,on_delete=models.PROTECT)   
    voucher_code = models.CharField(max_length=4, unique=True ,blank=True,editable=False,help_text="Will be generated automatically if left blank")#        editable=False  # Prevent manual edits
    name = models.CharField(max_length=100)
    description = models.TextField(blank=True, null=True)
    #discount_type = models.CharField(max_length=1,choices=DISCOUNT_TYPE_CHOICES,default="P")
    discount_value_sessions = models.IntegerField(# without discount_type fixed amount this is percentage 
        #max_digits=3, 
        #decimal_places=2,
        #if discount_type fixed amount applied , then max validator will be removed 
        validators=[MinValueValidator(0),MaxValueValidator(100)]
    )
    discount_value_visits = models.IntegerField(validators=[MinValueValidator(0),MaxValueValidator(100)])
    max_sessions = models.PositiveIntegerField(default=0)
    max_visits = models.PositiveIntegerField(default=0)
    #min_order_amount = models.DecimalField( max_digits=10, decimal_places=2, default=0.00 ) advanced level 
    # start_date and end_date replaced by validity_days 
    #start_date = models.DateTimeField(blank=True, null=True)
    #end_date = models.DateTimeField(blank=True, null=True)
    validity_days = models.IntegerField(default=1)
    #max_uses = models.PositiveIntegerField(blank=True, null=True) advanced level 
    # current_uses = models.PositiveIntegerField(default=0) can made from query 
    is_active = models.BooleanField(default=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_by = models.ForeignKey(User,related_name="Voucher_created_by",on_delete=models.PROTECT)

    class Meta:
        indexes = [
            models.Index(fields=['voucher_code']),
            #models.Index(fields=['is_active', 'end_date']),
        ]
    def save(self, *args, **kwargs):
        if not self.voucher_code:
            self.voucher_code = VoucherCodeManager.get_code()
        super().save(*args, **kwargs)
    def __str__(self):
        return f"{self.name} ({self.voucher_code})"


class Campaign(models.Model):
    STATUS_CHOICES = [
        ('draft', 'Draft'),
        ('running', 'Running'),
        ('completed', 'Completed'),
        ('paused', 'Paused'),
    ]
    name = models.CharField(max_length=100)
    msg_prefix = models.CharField(max_length=100, null=True, blank=True)
    voucher = models.ForeignKey(Voucher, on_delete=models.PROTECT)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    filter_start_date = models.DateField(null=True, blank=True)
    filter_end_date = models.DateField(null=True, blank=True)
    filter_pt_status = models.CharField(max_length=250, null=True, blank=True)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='draft')
    total_recipients = models.PositiveIntegerField(default=0)
    success_count = models.PositiveIntegerField(default=0)
    failed_count = models.PositiveIntegerField(default=0)
    
    def __str__(self):
        return self.name


class PatientVoucher(models.Model):
    MSG_STATUS_CHOICES = [
        ('pending', 'Pending'),
        ('sent', 'Sent'),
        ('failed', 'Failed'),
        ('delivered', 'Delivered'),
    ]
    voucher = models.ForeignKey( Voucher,related_name='PatientVoucher_voucher',on_delete=models.PROTECT )
    patient =  models.ForeignKey(Patient,related_name="PatientVoucher_patient",on_delete=models.PROTECT,)
    # old_code : first letter - patient_id - second letter - discount_type - 3rd letter - discount_amount - 4th letter validity days 
    voucher_code_assigned = models.CharField(max_length=15,blank=True, null=True)#mix of original code 
    discount_amount_sessions_assigned = models.IntegerField(blank=True, null=True)# loaded from original voucher , keep here in case if voucher changed its data 
    discount_amount_visits_assigned = models.IntegerField(blank=True, null=True)# loaded from original voucher , keep here in case if voucher changed its data 
    max_sessions = models.PositiveIntegerField(default=0)
    max_visits = models.PositiveIntegerField(default=0)
    
    discount_type_assigned = models.CharField(max_length=1,choices=DISCOUNT_TYPE_CHOICES,default="P")# loaded from original voucher , keep here in case if voucher changed its data
    expires_at = models.DateField(blank=True, null=True)
    issuer_notes = models.CharField(max_length=50,blank=True, null=True)#ملاحظات اللى عمل الخصم بتظهر فى صفحة الادارة 
    is_wa_sent = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    created_by = models.ForeignKey(User,related_name="PatientVoucher_created_by",on_delete=models.PROTECT)
    #after usage data 
    pt_session =  models.ForeignKey(PatientSession,blank=True, null=True,related_name="PatientVoucher_patient",on_delete=models.PROTECT,)
    is_used_session = models.BooleanField(default=False)
    visits_uses_count = models.IntegerField(default=0)#when add new PtVoucherVisits add one 
    
    campaign = models.ForeignKey(Campaign, related_name='messages', on_delete=models.PROTECT,blank=True, null=True,)
    msg_status = models.CharField(max_length=20, choices=MSG_STATUS_CHOICES, default='pending')
    try_send_at = models.DateTimeField(null=True, blank=True)
    attempts = models.PositiveIntegerField(default=0) 
    #used_at = models.DateTimeField(blank=True, null=True)
    #used_discount_amount = models.IntegerField(blank=True, null=True)
    # class Meta:
    #     constraints = [
    #         models.UniqueConstraint(
    #             fields=['voucher_item', 'client'],
    #             name='unique_client_voucher'
    #         )
    #     ]
    #     indexes = [
    #         models.Index(fields=['client', 'is_used', 'expires_at']),
    #         models.Index(fields=['voucher_item', 'is_used']),
    #     ]

    def __str__(self):
        return f"{self.patient} - {self.voucher}"
    
from django.db.models.signals import post_save,pre_save
from django.dispatch import receiver

@receiver(pre_save, sender=PatientVoucher)
def PatientVoucher_pre_save(sender, instance, **kwargs):
    #previous_state
    import pdb
    #pdb.set_trace()
    campaign = instance.campaign
    if instance.attempts == 1 :
        if instance.is_wa_sent and instance.msg_status == 'sent' : 
            campaign.success_count = campaign.success_count + 1 
            campaign.save()
        elif not instance.is_wa_sent and instance.msg_status == 'failed' : 
            campaign.failed_count = campaign.failed_count + 1
            campaign.save()
    


    # """
    # Pre-save signal handler for PatientVoucher model
    # """
    # try:
    #     # Get previous state if the instance exists in DB
    #     if instance.campaign:
    #         old_instance = PatientVoucher.objects.get(pk=instance.pk)
    #         is_wa_sent_before = old_instance.is_wa_sent
    #     else:
    #         is_wa_sent_before = False
            
    #     # Your logic here using is_wa_sent_before
    #     # For example:
    #     if instance.is_wa_sent and not is_wa_sent_before:
    #         print(f"WA message status changed to sent for {instance}")
            
    # except PatientVoucher.DoesNotExist:
    #     # Handle case where instance doesn't exist yet
    #     pass

from brvisitApp.models import PatientVisit
class PtVoucherVisits(models.Model):
    pt_voucher =  models.ForeignKey(PatientVoucher,blank=True, null=True,related_name="PtVoucherVisits_pt_voucher",on_delete=models.PROTECT,)
    pt_visit =  models.ForeignKey(PatientVisit,blank=True, null=True,related_name="PtVoucherVisits_pt_visit",on_delete=models.PROTECT,)


from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from django.core.exceptions import ValidationError
from .models import PatientVoucher, PtVoucherVisits

@receiver(post_save, sender=PtVoucherVisits)
def PtVoucherVisits_post_save(sender, instance, created, **kwargs):
    """
    Update visits_uses_count when PtVoucherVisits is created
    """
    if created and instance.pt_voucher:
        voucher = instance.pt_voucher
        # Check if adding would exceed max_visits (post-save validation)
        # if voucher.visits_uses_count >= voucher.max_visits:
        #     # Delete the newly created instance if it exceeds max visits
        #     instance.delete()
        #     raise ValidationError(
        #         f"Cannot create visit - voucher {voucher.voucher_code_assigned} "
        #         f"has reached its maximum visits ({voucher.max_visits})"
        #     )
        # Increment the count
        voucher.visits_uses_count += 1
        voucher.save()

@receiver(post_delete, sender=PtVoucherVisits)
def PtVoucherVisits_post_delete(sender, instance, **kwargs):
    """
    Decrement visits_uses_count when PtVoucherVisits is deleted
    """
    if instance.pt_voucher and instance.pt_voucher.visits_uses_count > 0:
        voucher = instance.pt_voucher
        voucher.visits_uses_count -= 1
        voucher.save()


'''
class CampaignMessage(models.Model):
    MSG_STATUS_CHOICES = [
        ('pending', 'Pending'),
        ('sent', 'Sent'),
        ('failed', 'Failed'),
        ('delivered', 'Delivered'),
    ]
    
    campaign = models.ForeignKey(Campaign, related_name='messages', on_delete=models.PROTECT)
    patient = models.ForeignKey(Patient, on_delete=models.SET_NULL, null=True, blank=True)
    phone_number = models.CharField(max_length=20,null=True, blank=True)
    message = models.TextField()
    msg_status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='pending')
    try_send_at = models.DateTimeField(null=True, blank=True)
    is_wa_sent = models.BooleanField(default=False)
    #attempts = models.PositiveIntegerField(default=0)
    
    class Meta:
        ordering = ['-try_send_at']
    
    def __str__(self):
        return f"{self.campaign.name} - {self.phone_number}"

'''